@@ -1,4 +1,4 @@
-{-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE DeriveFunctor, TypeSynonymInstances, FlexibleInstances, TypeFamilies #-}
 -- |
 -- The `build` function can be used to construct multi-line string literals in
 -- a monadic way:
@@ -21,10 +21,18 @@ module Data.String.Builder (
 , BuilderM
 ) where
 
-import Data.String
+import           Control.Applicative
+import           Control.Monad
+import           Data.Monoid
+import           Data.String
 
 -- | A writer monad for string literals.
 data BuilderM a = BuilderM a ShowS
+  deriving Functor
+
+instance Applicative BuilderM where
+  pure = return
+  (<*>) = ap
 
 instance Monad BuilderM where
   return a            = BuilderM a id
@@ -33,6 +41,10 @@ instance Monad BuilderM where
 
 type Builder = BuilderM ()
 
+instance Monoid Builder where
+  mempty = return ()
+  a `mappend` b = a >> b
+
 -- | Add a literal string.
 literal :: String -> Builder
 literal = BuilderM () . showString
